"""Constants used by mutation application logic."""

ARG_TO_POS_MAP = {
    'in_channels': 0, 'out_channels': 1,
    'in_features': 0, 'out_features': 1,
    'num_features': 0,
    'normalized_shape': 0
}
